#!/usr/bin/perl -w

use strict;
use Getopt::Std;
use DBI;
use Net::DNS;

my $options = {};
getopts ('hu:p:', $options);
if ($options->{h} || @ARGV < 4) {
	$0 =~ m|([^/]+)$|;
	print "Usage is: $1 [-h] [-u dbuser] [-p dbpass]] DSN uname ip url\n";
	exit;
}

my ($dsn, $uname, $ip, $url, $cats) = @ARGV;
my $anames = {0 => 'ACCEPT', 1 => 'REJECT', 2 => 'REDIRECT'};

my @bytes = split /\./, $ip;
$ip = ((($bytes [0] << 8) + $bytes [1] << 8) + $bytes [2] << 8) + $bytes [3];
$url =~ m|^\w+://([^/]+)|;
my $host = $1;
my $res = new Net::DNS::Resolver (nameservers => ['81.176.72.86'], recurse => 0);
my $query = $res->query ($host, "TXT");
if ($query) {
	for (grep $_->type eq 'TXT', $query->answer) {
		my $tmp = $_->rdatastr;
		$tmp =~ s/^"(.*)"$/$1/g;
		$tmp =~ s/,/:/g;
		$cats .= "$tmp:";
	}
} else {
	$cats = '';
}

my $dbh = DBI->connect ($dsn, $options->{u}, $options->{p});
if (!$dbh) {
	print STDERR "Error opening database '$dsn': ", DBI->errstr, "\n";
	exit (1);
};

my ($rname, $action) = check ($uname, $ip, $url, $cats);

print "Uname: $uname\n";
print "IP: ", join ('.', @bytes), "\n";
print "URL: $url\n";
print "Host: $host\n";
print "Role: $rname\n";
print "Cats: $cats\n";
print "Action: $anames->{$action}\n";

sub check {
	my ($uname, $ip, $url, $cats) = @_;
	my ($role, $rname);

	my $roles = $dbh->selectall_arrayref ("SELECT u.addr, u.mask, u.rolid, r.rname FROM users u, roles r WHERE u.uname = " . $dbh->quote ($uname) . " AND u.rolid = r.rolid ORDER BY mask DESC");
	for (@$roles) {
		if (($ip & $_->[1]) == $_->[0]) {
			$role = $_->[2];
			$rname = $_->[3];
			last;
		}
	}

	if (!$role) {
		my $roles = $dbh->selectall_arrayref ("SELECT u.addr, u.mask, u.rolid, r.rname FROM users u, roles r WHERE u.uname = '' AND u.rolid = r.rolid ORDER BY mask DESC");
		for (@$roles) {
			if (($ip & $_->[1]) == $_->[0]) {
				$role = $_->[2];
				$rname = $_->[3];
				last;
			}
		}
	}

	return ('UNKNOWN', 1) if !$role;

	my $lists = $dbh->selectall_arrayref ("SELECT l.url, l.action FROM lists l, hierarchy h WHERE h.child = $role AND h.parent = l.rolid ORDER BY rolid, stars, length (l.url)");
	print "url: $_->[0], action: $anames->{$_->[1]}\n" for @$lists;

	# check blacklist
	for (grep $_->[1] > 0, @$lists) {
		return ($rname, $_->[1]) if $url =~ /^$_->[0]$/;
	}

	# check whitelist
	for (grep $_->[1] == 0, @$lists) {
		return ($rname, $_->[1]) if $url =~ /^$_->[0]$/;
	}

	return ($rname, 0) if !$cats;

	my $perms = $dbh->selectall_arrayref ("SELECT p.cat, p.action FROM perms p, hierarchy h WHERE h.child = $role AND h.parent = p.rolid ORDER BY rolid");
#	print "cat: $_->[0], action: $anames->{$_->[1]}\n" for @$perms;

	$cats = [split /:/, $cats];

	# check REJECT and REDIRECT perms
	for my $perm (grep $_->[1] > 0, @$perms) {
		return ($rname, $perm->[1]) if grep $perm->[0] == $_, @$cats;
	}

	# check ACCEPT perms
	for my $perm (grep $_->[1] == 0, @$perms) {
		return ($rname, $perm->[1]) if grep $perm->[0] == $_, @$cats;
	}

	return ($rname, 0);
}
