#!/usr/bin/perl -w

use strict;
use Digest::MD5 qw(md5_base64);
use CGI qw(header *html *div font table caption TR th td a b hr cookie remote_addr user_agent param url *form textfield password_field submit hidden);
use DBI;
require './config.ph';

if (!$::DSN) {
	print
	header (-charset => 'koi8-r'),
	start_html,
	font ({-size => '+1', -color => 'red'}, '     .   config.ph'),
	end_html;
	exit;
}

use constant	AdminNamePattern => '^\w+$';

if (grep (defined $_, map cookie ($_), qw(aname auth)) != 2 ) {	# Check cookie
	print header (-location => 'login.cgi');
	exit;
}

my ($err, $rights, $res);
my $dbh = DBI->connect ($::DSN, $::DBUser, $::DBPass, {ShowErrorStatement => 1, AutoCommit => 0});
if (!$dbh) {
	$err = '    : ' . DBI->errstr;
} else {
	my $u = $dbh->selectrow_arrayref ("SELECT passwd, cookie, type FROM admins WHERE aname = " . $dbh->quote (cookie ('aname')));
	if ($dbh->errstr) {
		$err = '    : ' . $dbh->errstr;
	} elsif ($u && cookie ('auth') eq md5_base64 ($u->[1], remote_addr, user_agent, cookie ('aname'), $u->[0])) {
		$rights = $u->[2];
	} else {
		if ($u) {
			$dbh->do ("UPDATE admins SET cookie = '' WHERE aname = " . $dbh->quote (cookie ('aname')));
			$dbh->commit;
		}
		print header (-location => 'login.cgi');
		exit;
	}
}

print
header (-charset => 'koi8-r'),
start_html (-title => ' ', -encoding => 'koi8-r'),
start_div ({-align => 'center'});
END {
	$dbh->disconnect if $dbh;
	print
	$err ? font ({-size => '+1', -color => 'red'}, $err) : '',
	hr,
	a ({-href => 'main.cgi'}, '   '),
	end_div,
	end_html;
}

exit if $err;

if (!defined param ('action') || !grep param ('action') eq $_, qw(new passwd delete)) {
	$err = 'oe ';
	exit;
}

if (param ('action') eq 'new') {
	if ($rights ne 'g') {
		$err = '       ';
		exit;
	}
	if (param ('save')) {
		if (!defined param ('aname')) {
			$err = '   ';
		} elsif (param ('aname') !~ /@{[AdminNamePattern]}/) {
			$err = '  ';
		} else {
			$res = $dbh->selectrow_arrayref ("SELECT count(*) FROM admins WHERE aname = " . $dbh->quote (param ('aname')));
			if ($dbh->errstr) {
				$err = '    : ' . $dbh->errstr;
			} elsif ($res->[0] != 0) {
				$err = '     ';
			} elsif (grep (!defined $_ || !length $_, map param ($_), qw(passwd1 passwd2)) || param ('passwd1') ne param ('passwd2')) {
				$err = '  ';
			} else {
				$dbh->do ("INSERT INTO admins (aname, passwd, type) VALUES (" . $dbh->quote (param ('aname')) . ', ' . $dbh->quote (param ('passwd1')) . ", 'c')");
				$err = '    : ' . $dbh->errstr if $dbh->errstr;
			}
		}
	}
	if (param ('save') && !$err) {
		$dbh->commit;
		print font ({-size => '+1'}, '  ');
	} else {
		print font ({-size => '+1', -color => 'red'}, $err), hr if $err;
		undef $err;
		print
		start_form ('post', url (-relative => 1)),
		table ({-border => 1},
			caption (font ({-size => '+1'}, '  ')),
			TR (th (' '), td (textfield ('aname', undef, 20, 20))),
			TR (th (''), td (password_field ('passwd1', undef, 20, 20))),
			TR (th ('ݣ '), td (password_field ('passwd2', undef, 20, 20))),
			TR (th ({-colspan => 2, align => 'center'}, submit ('save', '')))
		),
		hidden ('action'),
		end_form;
	}
} else {
	if (!defined param ('value')) {
		$err = '   ';
		exit;
	}

	if (param ('value') !~ /@{[AdminNamePattern]}/) {
		$err = '  ';
		exit;
	}

	$res = $dbh->selectrow_arrayref ("SELECT type FROM admins WHERE aname = " . $dbh->quote (param ('value')));
	if ($dbh->errstr) {
		$err = '    : ' . $dbh->errstr;
		exit;
	}

	if (!$res) {
		$err = '  ';
		exit;
	}

	if (param ('action') eq 'passwd') {
		if (cookie ('aname') ne param ('value') && ($rights ne 'g' || $res->[0] eq 'g')) {
			$err = '       ';
			exit;
		}

		if (param ('save')) {
			if (grep (!defined $_ || !length $_, map param ($_), qw(passwd1 passwd2)) || param ('passwd1') ne param ('passwd2')) {
				$err = '  ';
			} else {
				$dbh->do ("UPDATE admins SET passwd = " . $dbh->quote (param ('passwd1')) . " WHERE aname = " . $dbh->quote (param ('value')));
				$err = '    : ' . $dbh->errstr if $dbh->errstr;
			}
		}
		if (param ('save') && !$err) {
			$dbh->commit;
			print font ({-size => '+1'}, '  ');
		} else {
			print font ({-size => '+1', -color => 'red'}, $err), hr if $err;
			undef $err;
			print
			start_form ('post', url (-relative => 1)),
			table ({-border => 1},
				caption (font ({-size => '+1'}, '  ', b (param ('value')))),
				TR (th (''), td (password_field ('passwd1', undef, 20, 20))),
				TR (th ('ݣ '), td (password_field ('passwd2', undef, 20, 20))),
				TR (th ({-colspan => 2, align => 'center'}, submit ('save', '')))
			),
			hidden ('action'),
			hidden ('value'),
			end_form;
		}
	} elsif (param ('action') eq 'delete') {
		if ($rights ne 'g' || $res->[0] eq 'g') {
			$err = '      ';
			exit;
		}

		if (param ('save')) {
			$dbh->do ("DELETE FROM admins WHERE aname = " . $dbh->quote (param ('value')));
			$err = '    : ' . $dbh->errstr if $dbh->errstr;
		}

		if (param ('save') && !$err) {
			$dbh->commit;
			print font ({-size => '+1'}, '', b (param ('value')), ' ̣');
		} else {
			print font ({-size => '+1', -color => 'red'}, $err), hr if $err;
			undef $err;
			print
			start_form ('post', url (-relative => 1)),
			table ({-border => 1},
				caption (font ({-size => '+1'}, ' ', b (param ('value')))),
				TR (th ('   , ', submit ('save', '')))
			),
			hidden ('action'),
			hidden ('value'),
			end_form;
		}
	}
}
